exception Error of string list 
let syntax_error err mes =  raise (Error [err; mes])
let handle_syntax_error f x = 
  try f x with 
   Error err -> 
     prerr_newline();
     prerr_endline  (String.concat " " ("Syntax error:" :: err));
     exit 2
;;

let status_regexp = 
  Str.regexp
    "\\([^\t ]+\\)[ \t]+\\([^\t ]+\\)[ \t]+\\(.*[^ \t]\\)?[ \t]*\r", 
  [1; 2; 3];;
let header_regexp = 
  Str.regexp
    "\\([^\t :]+\\)[ \t]*:[ \t]*\\([^\r]*\\)\r", 
  [1; 2];;
let chunk_regexp = 
  Str.regexp
    "\\([A-Za-z0-9]+\\)\\([ \t]+.*\\)?\r",
    [1];;
let token_regexp = 
  Str.regexp
    "[ \t]+";;


let handle_group str k = try Str.matched_group k str with Not_found -> "";;
let match_string (regexp, groups) str =
  if Str.string_match regexp str 0 then
    List.map (handle_group str) groups
  else []

let parse_status chan = 
  let line = input_line chan in
  match match_string status_regexp line with
  | _ :: code_string :: mes :: _ -> 
      let code = 
        try int_of_string code_string 
        with Failure _ -> syntax_error "Ill formed code" code_string in 
      code, mes
  | _ -> syntax_error "Ill formed status" line

let parse_command chan = 
  let line = input_line chan in
  match match_string status_regexp line with
  | command :: uri :: proto :: _ ->
      String.uppercase command, uri, String.uppercase proto
  | _ -> syntax_error "Ill formed command" line

let parse_header chan = 
  let rec parse_line line = 
    if line = "\r" then []
    else
      match match_string header_regexp line with 
        field :: rest :: _ -> 
          let rec parse line =
            let next = input_line chan in
            if next = "" then syntax_error "Ill formed header" "empty line"
            else if next.[0] = ' ' || next.[0] = '\t' then 
              parse (line ^ next) 
            else 
                (String.lowercase field, line) :: parse_line next in
          parse rest 
      | _ -> syntax_error "Ill formed header" line in
  parse_line (input_line chan);;

let tokenize line = 
  List.map String.lowercase (Str.split token_regexp line);;

let parse_response chan  =
  let status = parse_status chan in
  let header = parse_header chan in 
  status, header

let parse_request chan = 
  let command = parse_command chan in
  let header = parse_header chan in 
  command, header

let parse_chunk chan = 
  let line = input_line chan in
  match match_string chunk_regexp line with
    size :: _ -> int_of_string ("0x" ^ size)
  | _ -> syntax_error "Ill formed header" line;;

let parse_crlf chan_in = 
  if input_char chan_in = '\r' && input_char chan_in = '\n' then ()
  else syntax_error "Ill formed header" "Expected CRLF";;

let tokenize str = 
  List.map String.lowercase (Str.split token_regexp str);;

