#302 "general.ltx"
(* $Id: general.ltx,v 1.10 2008/11/21 10:31:20 remy Exp $ *)

(** miscelleaneous functions for the Unix library *)

open Sys
open Unix

(** {6 Finalization} *)

val try_finalize : ('a -> 'b) -> 'a -> ('c -> unit) -> 'c -> 'b
(** [try_finalize f x g y] applies the main code [f] to [x] and
     the result after having executed the finalization 
    code [g] applied to [y]. If the main code raises the exception
    [exn] the finalization finalization code and [exn] is raised.
    If the finalization code itself fails, the exception
    return is always the one of the finalization code. *)


#515 "fich.ltx"
(*** Directory iterator *)
val iter_dir : (string -> 'a) -> string -> unit
(** [iter_dir f d] opens path [d] as a directory and iterates the 
function [f] over all its entries *)


#373 "sign.ltx"
val restart_on_EINTR : ('a -> 'b) -> 'a -> 'b
(** [restart_on_EINTR f x] calls [f x] repeatedly until it does not fails
with [EINTR] *)
#395 "sign.ltx"
val free_children : int -> unit
 (** [free_children signal] free all zombies of the current processus, 
     discarding their exit status. *)

#451 "sign.ltx"
val system : string -> process_status
(** [system cmd] behaves as [Unix.system cmd] except that [sigchld] is
blocked and [sigint] and [sigquit] are ignored during the excecution 
of the command [cmd] *)

val exec_as_system : ('a -> process_status) -> 'a -> process_status
(** [system exec args] behaves as [system cmd] except that it takes as
arguments a function [exec] and arguments [args] to be passed to [exec]
to start in the new process. In particular, [system] is an abbreviation for 
[exec_as_system (execv "/bin/sh") \[|"-c"; cmd; |\] v]. *)

#1353 "comm1.ltx"
val really_write : file_descr -> string -> int -> int -> unit
(** as [single_write] but restarts on [EINTR] until all bytes have been
written. When an error occurs, some unknown number of bytes may have been
written. Hence, an error should in general be considered as fatal. *)

#406 "comm2.ltx"
val retransmit : file_descr -> file_descr -> unit
(* [retransmit fdin fdout] copy the contain of fdin into fdout, until 
 it receives end of file in fdin. It does not close descriptors at the end *)

#557 "comm2.ltx"
val install_tcp_server_socket : Unix.sockaddr -> Unix.file_descr
(** [install_tcp_server_socket sockaddr] creates a socket in the internet
domain,  binds it to the address [sockaddr], listens to it, and returns
it. *) 

#573 "comm2.ltx"
val tcp_server :
  (file_descr -> file_descr * sockaddr -> 'a) -> sockaddr -> unit
(** [tcp_server f addr] installs the tcp service [f] at the internet domain
    address  [addr]. For each connection to the service, the
    function [f] receives the server's socket and the client's
    socket-address pair as parametters. It is the responsability of  [f] to
    close the client connection when done. *) 

#627 "comm2.ltx"
val sequential_treatment : 
 file_descr -> (file_descr * sockaddr -> unit) -> 
 file_descr * sockaddr -> unit
(** [sequential_treatment server service client] runs [service] provided
       on server for one client. The server is given by its socket [socket]
       and the client is given by its socket-address pair [client]. After
       initialization the service is performed by applying [service] to
       [client]. The treatment is sequential, that is, the function only
       returns when the service is completed. *)

#657 "comm2.ltx"
val fork_treatment : 
  file_descr -> (file_descr * sockaddr -> unit) -> 
  file_descr * sockaddr -> unit
(** same as [sequential_treament] but the treatment is concurrently
  performed by a forked child process of the server. The parent process
  will have to free the child when completed. *)

#693 "comm2.ltx"
val double_fork_treatment : 
 file_descr -> (file_descr * sockaddr -> unit) -> 
 file_descr * sockaddr -> unit
(** same as [fork_treament] but the treatment is performed after a double
    fork. The forked process will be freed automatically on
    completion. *)  

